absFilter <-
function(bamPath, bamFile, asPath, asFile, absPath, absSim, absGlobal = 5, absLocal = 0.05) {
    cat(paste("<<", bamPath, "\n"))
    cat(paste("<<", bamFile, "\n"))
    cat(paste("<<", asPath, "\n"))
    cat(paste("<<", asFile, "\n"))

    asDat <- read.table(paste(asPath, asFile, sep = ""), sep = "\t", header = FALSE)

    ## count number of URSS per site
    void <- countUrss(bamPath, bamFile, asPath, asFile, absPath)
    asDatUrss <- read.table(paste(absPath, asFile, ".urss.txt", sep = ""), header = TRUE)

    ## KS-test based clonality coefficient estimation
    asDatUrssMean<- unlist(lapply(split(asDatUrss, asDatUrss$COUNT), function(x) if(nrow(x) >= 2) {mean(x[,3])} else {NA}))
    isValid <- as.numeric(names(asDatUrssMean)) <= 2000 & !is.na(asDatUrssMean)
    asDatUrssMeanFiltered <- asDatUrssMean[isValid]

    validLocal <- NA
    validGlobal <- NA
    validLocalAndGlobal <- NA
    if(length(asDatUrssMeanFiltered) >= 10) { ## min. 10 URSS bins required to estimate library clonality
        absSimMean <- t(do.call(rbind, lapply(absSim, function(x) x$mean)))
        rownames(absSimMean) <- 1:nrow(absSimMean)

        use <- rownames(absSimMean) %in% names(asDatUrssMeanFiltered)
        absSimMeanFiltered <- absSimMean[use,]

        absKsD <- c()
        for(i in 1:ncol(absSimMeanFiltered)) {
            absKsD[i] <- ks.test(asDatUrssMeanFiltered, absSimMeanFiltered[,i])$statistic
        }
        names(absKsD) <- colnames(absSimMeanFiltered)
        fitCoeff <- as.numeric(names(absKsD[which.min(absKsD)]))
        absSimFit <- absSimMean[,colnames(absSimMean) %in% fitCoeff]

        absFit <- absSim[names(absSim) %in% fitCoeff][[1]]
        absPvalue <- c()
        for(i in 1:nrow(asDatUrss)) {
            asDatUrssSub <- asDatUrss[i,]

            if(asDatUrssSub$COUNT > 2000) { ## max. simulated read depth
                asDatUrssSub$COUNT <- 2000
            }

            absFitSamples <- unlist(absFit$samples[asDatUrssSub$COUNT])
            absPvalue[i] <- sum(absFitSamples <= asDatUrssSub$NURSS)/length(absFitSamples)
        }

        validLocal <- absPvalue >= absLocal
        validGlobal <- asDatUrss$NURSS >= absGlobal
        validLocalAndGlobal <- validGlobal & validLocal
        asDatValid <- asDat[validLocalAndGlobal,]
        asDatNotValid <- asDat[!validLocalAndGlobal,]
    } else {
        cat(paste("Not enough sites to estimate library clonality.\n"))
        asDatValid <- asDat
        asDatNotValid <- NA
    }

    cat(paste("#tested sites:", nrow(asDat), "\n"))
    cat(paste("#unbiased sites:", nrow(asDatValid), "\n"))
    cat(paste("#biased sites: ", nrow(asDatNotValid), " (global: ", sum(!validGlobal), "; local: ", sum(!validLocal), ")", sep = "", "\n"))
    out <- list("aubs" = asDatValid, "abs" = asDatNotValid)
    return(out)
}
